@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2010 Open Source Applications Foundation.
+ * Copyright (c) 2004-2011 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -175,9 +175,7 @@ DECLARE_TYPE(SimpleDateFormat, t_simpledateformat, DateFormat,
 
 PyObject *wrap_DateFormat(DateFormat *format)
 {
-    if (format->getDynamicClassID() == SimpleDateFormat::getStaticClassID())
-        return wrap_SimpleDateFormat((SimpleDateFormat *) format, T_OWNED);
-
+    RETURN_WRAPPED_IF_ISINSTANCE(format, SimpleDateFormat);
     return wrap_DateFormat(format, T_OWNED);
 }
 
@@ -276,7 +274,7 @@ static int t_dateformatsymbols_init(t_dateformatsymbols *self,
     UnicodeString _u;
     Locale *locale;
     DateFormatSymbols *dfs;
-    char *type;
+    charsArg type;
 
     switch (PyTuple_Size(args)) {
       case 0:
@@ -292,7 +290,7 @@ static int t_dateformatsymbols_init(t_dateformatsymbols *self,
             self->flags = T_OWNED;
             break;
         }
-        if (!parseArgs(args, "c", &type))
+        if (!parseArgs(args, "n", &type))
         {
             INT_STATUS_CALL(dfs = new DateFormatSymbols(type, status));
             self->object = dfs;
@@ -302,7 +300,7 @@ static int t_dateformatsymbols_init(t_dateformatsymbols *self,
         PyErr_SetArgsError((PyObject *) self, "__init__", args);
         return -1;
       case 2:
-        if (!parseArgs(args, "Pc", TYPE_CLASSID(Locale),
+        if (!parseArgs(args, "Pn", TYPE_CLASSID(Locale),
                        &locale, &type))
         {
             INT_STATUS_CALL(dfs = new DateFormatSymbols(*locale, type, status));
@@ -515,7 +513,7 @@ static PyObject *t_dateformatsymbols_setAmPmStrings(t_dateformatsymbols *self,
     return PyErr_SetArgsError((PyObject *) self, "setAmPmStrings", arg);
 }
 
-DECLARE_RICHCMP(DateFormatSymbols, t_dateformatsymbols);
+DEFINE_RICHCMP(DateFormatSymbols, t_dateformatsymbols);
         
 static PyObject *t_dateformatsymbols_getLocalPatternChars(t_dateformatsymbols *self, PyObject *args)
 {
@@ -1062,7 +1060,7 @@ static PyObject *t_dateinterval_str(t_dateinterval *self)
     return PyUnicode_FromUnicodeString(&u);
 }
 
-DECLARE_RICHCMP(DateInterval, t_dateinterval);
+DEFINE_RICHCMP(DateInterval, t_dateinterval);
 
 
 /* DateIntervalInfo */
@@ -1186,7 +1184,7 @@ static PyObject *t_dateintervalinfo_getFallbackIntervalPattern(t_dateintervalinf
     return PyErr_SetArgsError((PyObject *) self, "getFallbackIntervalPattern", args);
 }
 
-DECLARE_RICHCMP(DateIntervalInfo, t_dateintervalinfo);
+DEFINE_RICHCMP(DateIntervalInfo, t_dateintervalinfo);
 
 
 /* DateIntervalFormat */
@@ -1257,16 +1255,7 @@ static PyObject *t_dateintervalformat_setDateIntervalInfo(t_dateintervalformat *
 static PyObject *t_dateintervalformat_getDateFormat(t_dateintervalformat *self)
 {
     DateFormat *format = (DateFormat *) self->object->getDateFormat()->clone();
-
-#if U_ICU_VERSION_HEX < 0x04060000
-    if (format->getDynamicClassID() == SimpleDateFormat::getStaticClassID())
-        return wrap_SimpleDateFormat((SimpleDateFormat *) format, T_OWNED);
-#else
-    if (dynamic_cast<SimpleDateFormat *>(format) != NULL)
-        return wrap_SimpleDateFormat((SimpleDateFormat *) format, T_OWNED);
-#endif
-
-    return wrap_DateFormat(format, T_OWNED);
+    return wrap_DateFormat(format);
 }
 
 static PyObject *t_dateintervalformat_parseObject(t_dateintervalformat *self,
@@ -1324,7 +1313,7 @@ static PyObject *t_dateintervalformat_createInstance(PyTypeObject *type,
     return PyErr_SetArgsError(type, "createInstance", args);
 }
 
-DECLARE_RICHCMP(DateIntervalFormat, t_dateintervalformat);
+DEFINE_RICHCMP(DateIntervalFormat, t_dateintervalformat);
 
 #endif
 
